from es import ServerVar, AddonInfo
from cfglib import AddonCFG
from spe import detourFunction, undetourFunction, makeObject, HookType, HookAction

info = AddonInfo()
info.name = 'Damage Modifier'
info.version = 1.0
info.basename = 'damage_modifier'
info.url = 'http://addons.eventscripts.com/addons/view/damage_modifier'
info.author = 'Dead Man Walker'

damage_types = {
  "GENERIC" : (0, "World"),
  "CRUSH" : (1<<0, "Crushed by a physics object (assumes physics force already caused by the object)"),
  "BULLET" : (1<<1, "Player was shot at"),
  "SLASH" : (1<<2, "Cut, clawed, stabbed"),
  "BURN" : (1<<3, "Heat burned"),
  "VEHICLE" : (1<<4, "Hit by a vehicle"),
  "FALL" : (1<<5, "Fell too far"),
  "BLAST" : (1<<6, "Explosive blast damage"),
  "CLUB" : (1<<7, "Crowbar, punch, headbutt"),
  "SHOCK" : (1<<8, "Electric shock"),
  "SNOIC" : (1<<9, "Sound pulse shockwave"),
  "ENERGYBEAM" : (1<<10, "Laser or other high energy beam"),
  "PREVENT_PHYSICS_FORCE" : (1<<11, "+Prevent a physics force"),
  "NERVEGIB" : (1<<12, "+Victim may not be gibbed upon death"),
  "ALWAYSGIB" : (1<<13, "+Victim will be gibbed upon death"),
  "DROWN" : (1<<14, "Drowning, see also DROWNRECOVER below"),
  "PARALYSE" : (1<<15, "Effect: Slows victim down"),
  "NERVEGAS" : (1<<16, "Nerve toxins, very bad"),
  "POISON" : (1<<17, "Blood poisoning, heals over time"),
  "RADIATION" : (1<<18, "Radiation exposure"),
  "DROWNRECOVER" : (1<<19, "Recovering from drowning"),
  "ACID" : (1<<20, "Toxic chemicals or acid burns"),
  "SLOWBURN" : (1<<21, "In an oven"),
  "REMOVENORAGDOLL" : (1<<22, "No ragdoll will be created, target will be quietly removed"),
  "PHYSGUN" : (1<<23, "Hit by manipulator"),
  "PLASMA" : (1<<24, "Shot by Cremator"),
  "AIRBOAT" : (1<<25, "Hit by the airboat's gun"),
  "DISSOLVE" : (1<<26, "Dissolving"),
  "BLAST_SURFACE" : (1<<27, "A blast on the surface of water that cannot harm things underwater"),
  "DIRECT" : (1<<28, "Direct damage? (Untested)"),
  "BUCKSHOT" : (1<<29, "Shotgun shot"),
  "HEADSHOT" : (1<<30, "+Display headshot symbol upon death")
}

def load():
   createCfg()
   detourFunction('OnTakeDamage', HookType.Pre, preventDamage)

def unload():
   undetourFunction('OnTakeDamage', HookType.Pre, preventDamage)

def es_map_start(event_var):
   createCfg()

def preventDamage(args):
   a = args[0]
   info = makeObject('CTakeDamageInfo', a)
   if cfg_damage_types == 1:
      return (HookAction.Override, 0)
   if cfg_damage_types == 0:
      return (HookAction.Continue, 0)
   for dmg_type_tuple in cfg_damage_types.values():
      if info.bitsDamageType & dmg_type_tuple[0]:
         info.flDamage *= dmg_type_tuple[1]
         break
   return (HookAction.Continue, 0)

def createCfg():
   global cfg_damage_types
   cfg = AddonCFG(ServerVar('eventscripts_gamedir') + '/cfg/damage_modifier.cfg')
   cfg.text('*************************************************************')
   cfg.text('*** Damage Modifier Config')
   cfg.text('*************************************************************')
   cfg.text(' ')
   cfg.text("Enter the damage types with their corresponding multiplicator.")
   cfg.text("Seperate each pair with a semicolon (;).")
   cfg.text("Use the following syntax:")
   cfg.text("<damage type>:<multiplicator>;<...>")
   cfg.text("Example:")
   cfg.text("HEADSHOT:0;DROWN:0.5;BURN:2")
   cfg.text("This prevents headshots at all, reduces drowning damage by half and doubles the burning damage.")
   damage_types_string = str(cfg.cvar("damage_types_string", "", "Leave blank for no change, type one to prevent any type of damage"))     
   cfg.text(' ')
   cfg.text(' ')
   cfg.text('List of Damage Types:')
   for dmg_name in damage_types:
      cfg.text("%s -- %s" %(dmg_name, damage_types[dmg_name][1]))
   cfg.write()
   cfg.execute()

   if damage_types_string == "1":
      cfg_damage_types = 1
   elif not damage_types_string:
      cfg_damage_types = 0
   else:
      damage_types_string = damage_types_string.strip()
      damage_types_list = damage_types_string.split(';') 
      cfg_damage_types = {}
      for dmg_type in damage_types_list:
         if not dmg_type:
            continue
         damage_split = dmg_type.split(':')
         if len(damage_split) != 2:
            raise IndexError, "Expected is '<damage type>:<multiplicator>'"
            continue
         if damage_split[0].upper() not in damage_types.keys():
            raise ValueError, "Damagetype '%s' does not exist!" %damage_split[0]
            continue
         try:
            damage_split[1] = float(damage_split[1])
         except ValueError, e:
            raise ValueError, e
            continue    
         cfg_damage_types[damage_split[0]] = (damage_types[damage_split[0]][0], damage_split[1])